@@ -159,6 +159,22 @@ QRectF qjackctlGraphItem::editorRect (void) const
 }
 
 
+// Path and bounding rectangle override.
+void qjackctlGraphItem::setPath ( const QPainterPath& path )
+{
+	m_rect = path.controlPointRect();
+
+	QGraphicsPathItem::setPath(path);
+}
+
+
+// Bounding rectangle accessor.
+const QRectF& qjackctlGraphItem::itemRect (void) const
+{
+	return m_rect;
+}
+
+
 //----------------------------------------------------------------------------
 // qjackctlGraphPort -- Port graphics item.
 
@@ -166,7 +182,8 @@ QRectF qjackctlGraphItem::editorRect (void) const
 qjackctlGraphPort::qjackctlGraphPort ( qjackctlGraphNode *node,
 	const QString& name, qjackctlGraphItem::Mode mode, uint type )
 	: qjackctlGraphItem(node), m_node(node),
-		m_name(name), m_mode(mode), m_type(type), m_index(0),
+		m_name(name), m_mode(mode), m_type(type),
+		m_index(node->ports().count()),
 		m_selectx(0), m_hilitex(0)
 {
 	QGraphicsPathItem::setZValue(+1.0);
@@ -266,7 +283,7 @@ void qjackctlGraphPort::setPortTitle ( const QString& title )
 	const QRectF& rect
 		= m_text->boundingRect().adjusted(0, +2, 0, -2);
 	path.addRoundedRect(rect, 5, 5);
-	QGraphicsPathItem::setPath(path);
+	/*QGraphicsPathItem::*/setPath(path);
 }
 
 
@@ -292,8 +309,7 @@ QPointF qjackctlGraphPort::portPos (void) const
 {
 	QPointF pos = QGraphicsPathItem::scenePos();
 
-	const QRectF& rect
-		= QGraphicsPathItem::boundingRect();
+	const QRectF& rect = itemRect();
 	if (m_mode == Output)
 		pos.setX(pos.x() + rect.width());
 	pos.setY(pos.y() + rect.height() / 2);
@@ -354,7 +370,7 @@ void qjackctlGraphPort::paint ( QPainter *painter,
 {
 	const QPalette& pal = option->palette;
 
-	const QRectF& port_rect = QGraphicsPathItem::boundingRect();
+	const QRectF& port_rect = itemRect();
 	QLinearGradient port_grad(0, port_rect.top(), 0, port_rect.bottom());
 	QColor port_color;
 	if (QGraphicsPathItem::isSelected()) {
@@ -643,7 +659,7 @@ qjackctlGraphNode::~qjackctlGraphNode (void)
 
 	// No actual need to destroy any children here...
 	//
-	//	QGraphicsPathItem::setGraphicsEffect(0);
+	//QGraphicsPathItem::setGraphicsEffect(nullptr);
 
 	//	delete m_text;
 	//	delete m_pixmap;
@@ -707,13 +723,22 @@ void qjackctlGraphNode::setNodeTitle ( const QString& title )
 {
 	const QFont& font = m_text->font();
 	m_text->setFont(QFont(font.family(), font.pointSize(), QFont::Bold));
-	m_text->setPlainText(title.isEmpty() ? m_name : title);
+	m_title = (title.isEmpty() ? m_name : title);
+
+	static const int MAX_TITLE_LENGTH = 29;
+	static const QString ellipsis(3, '.');
+
+	QString text = m_title;
+	if (text.length() >= MAX_TITLE_LENGTH  + ellipsis.length())
+		text = text.left(MAX_TITLE_LENGTH) + ellipsis;
+
+	m_text->setPlainText(text);
 }
 
 
 QString qjackctlGraphNode::nodeTitle (void) const
 {
-	return m_text->toPlainText();
+	return m_title;	// m_text->toPlainText();
 }
 
 
@@ -811,7 +836,7 @@ void qjackctlGraphNode::updatePath (void)
 	int wi, wo;
 	wi = wo = width;
 	foreach (qjackctlGraphPort *port, m_ports) {
-		const int w = port->boundingRect().width();
+		const int w = port->itemRect().width();
 		if (port->isOutput()) {
 			if (wo < w) wo = w;
 		} else {
@@ -827,7 +852,7 @@ void qjackctlGraphNode::updatePath (void)
 	int yi, yo;
 	yi = yo = height;
 	foreach (qjackctlGraphPort *port, m_ports) {
-		const QRectF& port_rect = port->boundingRect();
+		const QRectF& port_rect = port->itemRect();
 		const int w = port_rect.width();
 		const int h = port_rect.height() + 1;
 		if (type - port->portType()) {
@@ -850,7 +875,7 @@ void qjackctlGraphNode::updatePath (void)
 
 	QPainterPath path;
 	path.addRoundedRect(-width / 2, 0, width, height + 6, 5, 5);
-	QGraphicsPathItem::setPath(path);
+	/*QGraphicsPathItem::*/setPath(path);
 }
 
 
@@ -859,7 +884,7 @@ void qjackctlGraphNode::paint ( QPainter *painter,
 {
 	const QPalette& pal = option->palette;
 
-	const QRectF& node_rect = QGraphicsPathItem::boundingRect();
+	const QRectF& node_rect = itemRect();
 	QLinearGradient node_grad(0, node_rect.top(), 0, node_rect.bottom());
 	QColor node_color;
 	if (QGraphicsPathItem::isSelected()) {
@@ -927,17 +952,17 @@ qjackctlGraphConnect::qjackctlGraphConnect (void)
 
 	qjackctlGraphItem::setBackground(qjackctlGraphItem::foreground());
 
+#if 0//Disable drop-shadow effect...
 	const QPalette pal;
 	const bool is_darkest = (pal.base().color().value() < 24);
 	QColor shadow_color = (is_darkest ? Qt::white : Qt::black);
 	shadow_color.setAlpha(220);
-
 	QGraphicsDropShadowEffect *effect = new QGraphicsDropShadowEffect();
 	effect->setColor(shadow_color);
 	effect->setBlurRadius(is_darkest ? 4 : 8);
 	effect->setOffset(is_darkest ? 0 : 1);
 	QGraphicsPathItem::setGraphicsEffect(effect);
-
+#endif
 	QGraphicsPathItem::setAcceptHoverEvents(true);
 
 	qjackctlGraphItem::raise();
@@ -954,7 +979,7 @@ qjackctlGraphConnect::~qjackctlGraphConnect (void)
 
 	// No actual need to destroy any children here...
 	//
-	//	QGraphicsPathItem::setGraphicsEffect(0);
+	//QGraphicsPathItem::setGraphicsEffect(nullptr);
 }
 
 
@@ -1006,15 +1031,17 @@ void qjackctlGraphConnect::updatePathTo ( const QPointF& pos )
 {
 	const bool is_out0 = m_port1->isOutput();
 	const QPointF pos0 = m_port1->portPos();
-	const QPointF pos1 = (is_out0 ? pos0 : pos);
-	const QPointF pos4 = (is_out0 ? pos : pos0);
 
-	const qreal d2 = (is_out0 ? +2.0 : -2.0);
-	const QPointF pos1_2(pos1.x() + d2, pos1.y());
-	const QPointF pos3_4(pos4.x() - d2, pos4.y());
+	const QPointF d1(1.0, 0.0);
+	const QPointF pos1 = (is_out0 ? pos0 + d1 : pos  - d1);
+	const QPointF pos4 = (is_out0 ? pos  - d1 : pos0 + d1);
+
+	const QPointF d2(2.0, 0.0);
+	const QPointF pos1_2(is_out0 ? pos1 + d2 : pos1 - d2);
+	const QPointF pos3_4(is_out0 ? pos4 - d2 : pos4 + d2);
 
 	qjackctlGraphNode *node1 = m_port1->portNode();
-	const QRectF& rect1 = node1->boundingRect();
+	const QRectF& rect1 = node1->itemRect();
 	const qreal dx = pos3_4.x() - pos1_2.x();
 	const qreal dy = pos0.y() - node1->scenePos().y() - 0.5 * rect1.height();
 	const qreal y_max = rect1.height() + rect1.width();
@@ -1030,24 +1057,22 @@ void qjackctlGraphConnect::updatePathTo ( const QPointF& pos )
 	path.lineTo(pos1_2);
 	path.cubicTo(pos2, pos3, pos3_4);
 	path.lineTo(pos4);
+
 	const qreal arrow_angle = path.angleAtPercent(0.5) * M_PI / 180.0;
 	const QPointF arrow_pos0 = path.pointAtPercent(0.5);
-	path.lineTo(pos3_4);
-	path.cubicTo(pos3, pos2, pos1_2);
-	path.lineTo(pos1);
-	const qreal arrow_size = 12.0;
+	const qreal arrow_size = 8.0;
 	QVector<QPointF> arrow;
 	arrow.append(arrow_pos0);
 	arrow.append(arrow_pos0 - QPointF(
-		::sin(arrow_angle + M_PI / 2.3) * arrow_size,
-		::cos(arrow_angle + M_PI / 2.3) * arrow_size));
+		::sin(arrow_angle + M_PI / 2.25) * arrow_size,
+		::cos(arrow_angle + M_PI / 2.25) * arrow_size));
 	arrow.append(arrow_pos0 - QPointF(
-		::sin(arrow_angle + M_PI - M_PI / 2.3) * arrow_size,
-		::cos(arrow_angle + M_PI - M_PI / 2.3) * arrow_size));
+		::sin(arrow_angle + M_PI - M_PI / 2.25) * arrow_size,
+		::cos(arrow_angle + M_PI - M_PI / 2.25) * arrow_size));
 	arrow.append(arrow_pos0);
 	path.addPolygon(QPolygonF(arrow));
 
-	QGraphicsPathItem::setPath(path);
+	/*QGraphicsPathItem::*/setPath(path);
 }
 
 
@@ -1060,24 +1085,27 @@ void qjackctlGraphConnect::updatePath (void)
 void qjackctlGraphConnect::paint ( QPainter *painter,
 	const QStyleOptionGraphicsItem *option, QWidget */*widget*/ )
 {
-	if (QGraphicsPathItem::isSelected()) {
-		const QPalette& pal
-			= option->palette;
-		const QColor& color
-			= pal.highlight().color();
-		painter->setPen(QPen(color, 2));
-		painter->setBrush(color);
-	} else {
-		const QColor& color
-			= qjackctlGraphItem::foreground();
-		if (qjackctlGraphItem::isHighlight() || QGraphicsPathItem::isUnderMouse())
-			painter->setPen(color.lighter());
-		else
-			painter->setPen(color);
-		painter->setBrush(qjackctlGraphItem::background());
-	}
+	QColor color;
+	if (QGraphicsPathItem::isSelected())
+		color = option->palette.highlight().color();
+	else
+	if (qjackctlGraphItem::isHighlight() || QGraphicsPathItem::isUnderMouse())
+		color = qjackctlGraphItem::foreground().lighter();
+	else
+		color = qjackctlGraphItem::foreground();
 
-	painter->drawPath(QGraphicsPathItem::path());
+	const QPalette pal;
+	const bool is_darkest = (pal.base().color().value() < 24);
+	QColor shadow_color = (is_darkest ? Qt::white : Qt::black);
+	shadow_color.setAlpha(80);
+
+	const QPainterPath& path
+		= QGraphicsPathItem::path();
+	painter->setBrush(Qt::NoBrush);
+	painter->setPen(QPen(shadow_color, 3));
+	painter->drawPath(path.translated(+1.0, +1.0));
+	painter->setPen(QPen(color, 2));
+	painter->drawPath(path);
 }
 
 
@@ -1327,11 +1355,13 @@ bool qjackctlGraphCanvas::canDisconnect (void) const
 		switch (item->type()) {
 		case qjackctlGraphConnect::Type:
 			return true;
-		case qjackctlGraphPort::Type: {
-			qjackctlGraphPort *port = static_cast<qjackctlGraphPort *> (item);
-			if (!port->connects().isEmpty())
-				return true;
-			// Fall-thru...;
+		case qjackctlGraphNode::Type: {
+			qjackctlGraphNode *node = static_cast<qjackctlGraphNode *> (item);
+			foreach (qjackctlGraphPort *port, node->ports()) {
+				if (!port->connects().isEmpty())
+					return true;
+			}
+			// Fall thru...
 		}
 		default:
 			break;
@@ -1884,6 +1914,7 @@ void qjackctlGraphCanvas::connectItems (void)
 void qjackctlGraphCanvas::disconnectItems (void)
 {
 	QList<qjackctlGraphConnect *> connects;
+	QList<qjackctlGraphNode *> nodes;
 
 	foreach (QGraphicsItem *item, m_scene->selectedItems()) {
 		switch (item->type()) {
@@ -1893,19 +1924,28 @@ void qjackctlGraphCanvas::disconnectItems (void)
 				connects.append(connect);
 			break;
 		}
-		case qjackctlGraphPort::Type: {
-			qjackctlGraphPort *port = static_cast<qjackctlGraphPort *> (item);
-			foreach (qjackctlGraphConnect *connect, port->connects()) {
-				if (!connects.contains(connect))
-					connects.append(connect);
-			}
-			// Fall-thru...
-		}
+		case qjackctlGraphNode::Type:
+			nodes.append(static_cast<qjackctlGraphNode *> (item));
+			// Fall thru...
 		default:
 			break;
 		}
 	}
 
+	if (connects.isEmpty()) {
+		foreach (qjackctlGraphNode *node, nodes) {
+			foreach (qjackctlGraphPort *port, node->ports()) {
+				foreach (qjackctlGraphConnect *connect, port->connects()) {
+					if (!connects.contains(connect))
+						connects.append(connect);
+				}
+			}
+		}
+	}
+
+	if (connects.isEmpty())
+		return;
+
 //	m_selected_nodes = 0;
 //	m_scene->clearSelection();
 
